@@ -1,13 +1,16 @@
 __all__ = ['service', 'static']
 
 from _term import (
-    Hict, parse_term, parse_json, to_simple, to_json, to_str, is_str
+    Hict, is_str, to_str,
+    parse_term, parse_json,
+    to_simple, to_json,
 )
 from _crypto import (
     nacl_box, nacl_sign, nacl_verify,
     nacl_nonce, nacl_random, nacl_sha256,
     PrivateKey, PublicKey, Box, SigningKey
 )
+from _autoreload import deep_reload
 from _auth import hooks as auth
 from _state import get_state
 import _config as conf
@@ -36,6 +39,7 @@ try:
 except ImportError:
     import traceback
 
+
 # services registry -> see service() and lookup()
 registry = dict()
 
@@ -56,15 +60,16 @@ def decrypt(keys, sign, pad=None):
 
 def encrypt(state, box, term, pad=None):
     enc = box.encrypt(to_json(to_simple(term)), nacl_nonce())
+    assert not pad or len(enc) <= len(pad), 'short-pad'
     sign = nacl_sign(xor(enc, pad) if pad else enc, state.sseed)[0]
     return Hict(sign=b64encode(sign))
 
 
 def service(urlpath=None, auto_parse=True, static=False, accel=False,
             cache=False, force_download=False, content_disposition=False,
-            auto_close_file=True, safe=True, ssl=True, use_one_time_pad=True,
-            check_abusive=True, check_perm=True, postmortem_debug=False,
-            force_define=False, log_exc=True, **output):
+            auto_close_file=True, check_abusive=True, check_perm=True,
+            postmortem_debug=False, force_define=False, log_exc=True,
+            use_one_time_pad=True, safe=True, ssl=None, **output):
 
     for k, v in output.iteritems():
         if not (k and k.isalnum() and callable(v)):
@@ -80,14 +85,24 @@ def service(urlpath=None, auto_parse=True, static=False, accel=False,
     def decorator(f):
         srv_path = (urlpath or f.__name__).strip('/')
 
-        if srv_path in registry and not force_define:
-            raise exc.MultipleDefine(srv_path)
+        modules_autoreload = conf.MODULES_AUTORELOAD
+
+        if srv_path in registry:
+            if force_define is True or modules_autoreload:
+                logger.debug("redefine service '%s' %s" % (srv_path, f))
+            else:
+                raise exc.MultipleDefine(srv_path)
 
         @wraps(f)
         def wrapper(*args, **argd):
+
+            if modules_autoreload:
+                for module in modules_autoreload:
+                    deep_reload(module)
+
             _state_ = get_state()
 
-            if ssl is True and not _state_.ssl:
+            if ssl is not False and (conf.USE_SSL or ssl) and not _state_.ssl:
                 raise exc.RequireSSL(srv_path)
 
             domain = _state_.domain
@@ -316,7 +331,7 @@ def static(urlpath, root, index='index.html', **argd):
 static(conf.JFANERY_URLPATH, conf.JFANERY_DIRPATH)
 
 
-@service(ssl=False, safe=False, auto_parse=False, log_exc=False)
+@service('fanery/prelogin', ssl=False, safe=False, auto_parse=False, log_exc=False)             # noqa
 def prelogin(identity, **extra):
     state = get_state()
     origin = state.origin
@@ -364,7 +379,7 @@ def prelogin(identity, **extra):
                 enc=b64encode(nacl_random(184)))
 
 
-@service(ssl=False, safe=False, auto_parse=False, log_exc=False)
+@service('fanery/login', ssl=False, safe=False, auto_parse=False, log_exc=False)                # noqa
 def login(identity, sid, sign, force=False):
     state = get_state()
     origin = state.origin
@@ -408,7 +423,7 @@ def login(identity, sid, sign, force=False):
     raise exc.InvalidCredential
 
 
-@service(ssl=False, use_one_time_pad=False, auto_parse=False, log_exc=False)
+@service('fanery/gen_pad', ssl=False, use_one_time_pad=False, auto_parse=False, log_exc=False)  # noqa
 def gen_pad(box, call, pad, tstamp):
     state = get_state()
 
@@ -442,7 +457,7 @@ def gen_pad(box, call, pad, tstamp):
 
     # server call pad data
     pad = b64decode(pad)
-    size = len(pad) - 340
+    size = len(pad) - 410
     cvkey = SigningKey(cseed).verify_key
     cpad = pads[call][cid] = Hict(pad=nacl_random(size), call=call,
                                   skey=skey.encode(), sseed=sseed,
@@ -461,7 +476,7 @@ def gen_pad(box, call, pad, tstamp):
     return encrypt(state, box, data, pad)
 
 
-@service(ssl=False, auto_parse=False, log_exc=False)
+@service('fanery/logout', ssl=False, auto_parse=False, log_exc=False)
 def logout(*args, **argd):
     _state_ = get_state()
     _state_.expires = 0
