@@ -161,7 +161,7 @@ def handler(req):
             else:
                 headers['Content-Type'] = content_type = guess_type(ret)[0] or CONTENT_TYPE                 # noqa
 
-                if content_type == 'text/html':
+                if content_type.startswith('text/'):
                     headers.update(STRICT_HTML_HEADERS)
 
             return FileApp(ret, server=SERVER, headerlist=headers.items())
